// icontool by natrium42 <natrium@gmail.com>
// Tool to create NDS style icons from a 32x32 source BMP.

// Change Log:
// v1.0 - initial version

/*
 Description of NDS icon format:
 (1) 32 bytes of 0x00
 (2) 512 bytes of tile data
 (3) 32 bytes of palette (16 color entries)
 (4) 6*256 bytes of title text for 6 languages

 32 x 32 pixels divided into 16 tiles, each tile is 8 x 8 pixels
 Each pixel is a 4-bit color index.
 Palette contains 16 colors, each color is 16-bit BGR.
 Title text uses UTF-16 encoding, first language is English.
*/

#include "stdafx.h"
#include <iostream>
#include <stdio.h>
#include "raster.h"

typedef	unsigned char			u8;
typedef	unsigned short int		u16;
typedef	unsigned int			u32;

#define RGB16(r,g,b)			((r)+(g<<5)+(b<<10))

using namespace std;

u16 RGBQuadToRGB16(RGBQUAD quad) {
	u16 r = quad.rgbRed;
	u16 g = quad.rgbGreen;
	u16 b = quad.rgbBlue;

	r >>= 3; g >>= 3; b >>= 3;

	return RGB16(r, g, b);
}

int main(int argc, char* argv[]) {
	CRaster bmp;

	printf("-----------------------\n");
	printf("|    Icontool v1.0    |\n");
	printf("|    by natrium42     |\n");
	printf("-----------------------\n");

	if(argc < 3) {
		printf("Usage: icontool <input.bmp> <output.bin> title text\n");
		printf("       Separate lines with ';' for the title text.\n");
		return 0;
	}

	printf("Source image: %s\n", argv[1]);
	// open input BMP:
	int rval = bmp.LoadBMP(argv[1]);

	if(rval == 1) {
		printf("Error: Couldn't open file\n");
		exit(1);
	}
	if(rval == 2) {
		printf("Error: File is not BMP\n");
		exit(1);
	}

	printf("Size: %i x %i\n", bmp.Width, bmp.Height);
	printf("Bits per pixel: %i\n", bmp.BPP);
	printf("Colors used: %i\n", bmp.pbmi->bmiHeader.biClrUsed);

	if(bmp.Width != 32 || bmp.Height != 32) {
		printf("Error: Image should be 32 x 32\n");
		exit(1);
	}
	if(bmp.BPP != 8) {
		printf("Error: Image should use 8-bit indexed colors\n");
		exit(1);
	}

	FILE *fdata = fopen(argv[2], "wb");
	if (!fdata) {
		printf("Couldn't create output file!\n");
		exit(1);
	}

	// initial 32 bytes (0x00)
	for(int i = 0; i < 32; i++) {
		fputc(0x00, fdata);
	}

	// tile data (4 bit / tile, 4x4 total tiles)
	// 32 bytes per tile (in 4 bit mode)
	for(int row = 0; row < 4; row++) {
		for(int col = 0; col < 4; col++) {
			for(int y = 0; y < 8; y++) {
				for(int x = 0; x < 4; x++) {
					u8 b0 = bmp.Raster[row * 64 * 4 + y * 8 * 4 + (x * 2 + col * 8)];
					u8 b1 = bmp.Raster[row * 64 * 4 + y * 8 * 4 + (x * 2 + 1 + col * 8)];
					// two nibbles form a byte:
					fputc(((b1 & 0xF) << 4) | (b0 & 0xF), fdata);
				}
			}
		}
	}

	// palette
	for(int i = 0; i < 16; i++) {
		u16 entry = RGBQuadToRGB16(bmp.Palette[i]);
		fputc(entry & 0xFF, fdata);
		fputc((entry >> 8) & 0xFF, fdata);
	}

	// title text (6 languages)
	// first language (English) is filled with the remaining args
	// everything else is filled with 0x00
	int dataCount = 0;
	for(int i = 3; i < argc; i++) {
		char* token = (argv[i]);
		while (*token != 0) {
			char curr = *token;
			if(curr == ';')	// new line with ';' character
				curr = 0x0A;
			fputc(curr, fdata);
			fputc(0x00, fdata);
			dataCount += 2;
			token++;
		}
		if(i != argc) {
			fputc(' ', fdata);
			fputc(0x00, fdata);
			dataCount += 2;
		}
	}
	// fill the rest of the string with 0x00
	for(int i = 0; i < 256 - dataCount; i++) {
		fputc(0x00, fdata);
	}
	// fill the other 5 languages with 0x00
	for(int i = 0; i < 5 * 256; i++) {
		fputc(0x00, fdata);
	}

	fclose(fdata);

	return 0;
}

